/* DataCellN.cpp

DataCell derived class for numbers with implied decimal places

From Chapter 8 of
Using XML with Legacy Business Applications
Copyright (c) 2002 Addison-Wesley Professional

Author: Michael C. Rawlins
Rawlins EC Consulting
PMB 29
14 Canyon Creek Village
Richardson, TX 75080-1602
mike@rawlinsecconsulting.com
972-783-9573

This file is one of several utilities developed for the book.

All utilities developed for the book are free software; you can
redistribute and/or modify them under the terms of the GNU
General Public License, Version 2, as published by the Free
Software Foundation.

These utilities are distributed in the hope that they will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the book; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston,
MA  02111-1307  USA

The license is also available on the World Wide Web at
http://www.gnu.org/licenses/gpl.html

*/

//  Common header files
#include "BlasterIncludes.h"

//  Constructor with field number and grammar Element
DataCellN::DataCellN(int iFieldNumberIn,
    IXMLDOMElementPtr spGrammarIn) :
  DataCell(iFieldNumberIn, spGrammarIn) { }

//  Destructor
DataCellN::~DataCellN() {};


//  Convert from XML
int DataCellN::fromXML()
{
  char cDataType[16];
  char cImpliedDecimals[16];
  char * cTmp;

  int iImpliedDecimals, iDifferencePlaces,
    iDecimalPosition, iFractionalDigits, i;

  //  Data Type <- Call Grammar Element's getAttribute for
  //    "DataType"
  memset(cDataType,0,sizeof(cDataType));
  strcpy(cDataType,
      _bstr_t(spGrammar->getAttribute("DataType")));

  //  Implied Places <- Extract portion of Data Type following
  //    "N"
  memset(cImpliedDecimals,0,sizeof(cImpliedDecimals));
  cTmp = strchr(cDataType,'N');
  cTmp++;
  strcpy(cImpliedDecimals,cTmp);
  iImpliedDecimals = atoi(cImpliedDecimals);

  //  Difference Places <- Implied Places
  iDifferencePlaces = iImpliedDecimals;

  //  Trim spaces - If schema validation is done we won't have
  //  spaces in a decimal field, but we're not going to
  //  depend on them doing validation
  trim();
  if (iBufferLength == 0)
  {
    return SUCCESS;
  }

  //  Remove leading plus sign.  XML types allow it, but some
  //  legacy formats such as X12 don't; it is always implied.
  if ( cCellBuffer[0] == '+' )
  {
    strcpy(cCellBuffer,&(cCellBuffer[1]));
    cCellBuffer[iBufferLength] = 0;
    iBufferLength--;
  }
  
  
  //  Find decimal point in Number Depending on language
  //    features
  iDecimalPosition = 0;
  while ( (iDecimalPosition < iBufferLength) &&
          (cCellBuffer[iDecimalPosition] != '.' )  )
  {
    iDecimalPosition++;
  }

  //  IF Decimal Point present
  if (iDecimalPosition < iBufferLength)
  {
    //  Fractional Digits <- Digits to the right of the decimal
    //    point
    iFractionalDigits = iBufferLength - (iDecimalPosition + 1);

    //  Cell Buffer <- Shift fractional digits to the left to
    //    overlay the decimal point, and clear the last byte
    //  Decrement Cell Buffer Length
    strcpy(&(cCellBuffer[iDecimalPosition]),
        &(cCellBuffer[iDecimalPosition+1]));
    cCellBuffer[iBufferLength] = 0;
    iBufferLength--;

    //  Difference Places <- Implied Places - Fractional Digits
    iDifferencePlaces = iImpliedDecimals - iFractionalDigits;

    //  IF (Difference Places < 0)
    if (iDifferencePlaces < 0)
    {
      //  Cell Buffer <- Truncate rightmost Difference Places
      //    digits
      //  Buffer Length <- Buffer Length - Difference Places
      memset(&(cCellBuffer[iBufferLength + iDifferencePlaces]),
        0, (-iDifferencePlaces));
    } //  ENDIF
  }

  //  IF (Difference Places > 0)
  for (i = 0; i < iDifferencePlaces; i++)
  {
    //  Cell Buffer <- Append Difference Places number of
    //    zeroes
    memset(&(cCellBuffer[iBufferLength]), '0',
        iDifferencePlaces);
  } //  ENDIF

  iBufferLength = iBufferLength + iDifferencePlaces;
  //  Return success
  return SUCCESS;
}



//  Prepare output for legacy format
int DataCellN::prepareOutput()
{
  char cLength[5];
  char cSignCharacter;
  int i, iLength = 0;
  int iMinLength = 0;
  _variant_t vLength;

  //  Length <- Call Grammar Element's getAttribute for
  //    "Length"
  //  We may have Length or MinLength - With MSXML we get
  //  a variant and have to check if it is null
  vLength = spGrammar->getAttribute("Length");
  if (vLength.vt != VT_NULL)
  {
    strcpy(cLength,_bstr_t(vLength));
    if (strlen(cLength) > 0)
    {
      iLength = atoi(cLength);
      iMinLength = iLength;
    }
  }
  else
  {
    strcpy(cLength,
      _bstr_t(spGrammar->getAttribute("MinLength")));
    iMinLength = atoi(cLength);
  }

  //  Sign Character <- First character in Cell Buffer
  cSignCharacter = cCellBuffer[0];

  //  Call fillField, passing R for right justification, and
  //    Length
  fillField('R', iMinLength);


  //  IF (Sign Character = "+" or "-") and (First character in
  //    Cell Buffer = "0")
  if ( ((cSignCharacter == '+') || (cSignCharacter == '-')) &&
       (cCellBuffer[0] == '0') )
  {
    //  Dependent on language features, find Sign Character in
    //  Cell Buffer and swap with first character in Cell
    //  Buffer
    i = 1;
    while (cCellBuffer[i] != cSignCharacter)
    {
      i++;
    }
    cCellBuffer[0] = cSignCharacter;
    cCellBuffer[i] = '0';
  } //  ENDIF

  //  IF ( Length = 0 )
  if (iLength == 0)
  {
    //  Maximum Length <- Call Grammar Element's getAttribute
    //    for "MaxLength"
    memset(cLength,0,sizeof(cLength));
    strcpy(cLength,
        _bstr_t(spGrammar->getAttribute("MaxLength")));
    //  Length <- Maximum Length, converted from string
    iLength = atoi(cLength);
  } //  ENDIF

    //  IF (Cell Buffer Length > Length)
  if (iBufferLength > iLength)
  {
    //  Return error
    cerr << NO_TRUNCATION << endl
         << _bstr_t(spGrammar->getAttribute("ElementName"))
         <<  " " << iLength << endl
         << cCellBuffer << endl;
    return FAILURE;
  } //  ENDIF

  //  Return success
  return SUCCESS;

}


int DataCellN::toXML()
{
  //  Locals
  char cCellDataType[32];
  char cImpliedDecimals[32];
  char cWork[64];
  char cTarget[64];
  char * cTmp;

  int iImpliedDecimals, iLeft, iLength;

  memset(cCellDataType,0,sizeof(cCellDataType));
  memset(cImpliedDecimals,0,sizeof(cImpliedDecimals));
  memset(cWork,0,sizeof(cWork));
  memset(cTarget,0,sizeof(cTarget));

  //  Remove leading and trailing spaces and zeroes
  trimLeadingZeroes();

  //  Data Type <- Call Grammar Element's getAttribute for
  //    "DataType"
  strcpy(cCellDataType, 
    _bstr_t(spGrammar->getAttribute("DataType")) );

  //  Implied Places <- Extract portion of Data Type following
  //    "N"
  cTmp = strchr(cCellDataType,'N');
  cTmp++;
  strcpy(cImpliedDecimals,cTmp);
  iImpliedDecimals = atoi(cImpliedDecimals);

  //  IF Implied Places > 0
  if ((iImpliedDecimals == 0) || (iBufferLength == 0))
  {
    //  Return
    return SUCCESS;
  } //  ENDIF

  //  IF Cell Buffer Length after trimming is less than Implied
  //      places + 1
  iLength = strlen(cCellBuffer);
  if (iLength < (iImpliedDecimals + 1))
  {
    //  Pad to implied places with zeroes, plus a zero
    //    to the left of the decimal
    memset(cWork,'0',iImpliedDecimals);
    strcat(cWork,cCellBuffer);
  } //  ENDIF
  else
  {
    strcpy(cWork,cCellBuffer);
  }

  //  Cell Buffer <- Substring of Cell Buffer from 0 through
  //  (Length - Implied Places) + "." + leftmost Implied Places
  //    characters of Cell Buffer
  iLeft = strlen(cWork) - iImpliedDecimals;
  
  strncpy(cTarget,cWork,iLeft);
  strcat(cTarget,".");
  strcat(cTarget,&(cWork[iLeft]));

  //  Return success
  return putField(cTarget,strlen(cTarget));
}
