/* DataCellX12DT.cpp

DataCell derived class for X12 Date data type

From Chapter 9 of
Using XML with Legacy Business Applications
Copyright (c) 2002 Addison-Wesley Professional

Author: Michael C. Rawlins
Rawlins EC Consulting
PMB 29
14 Canyon Creek Village
Richardson, TX 75080-1602
mike@rawlinsecconsulting.com
972-783-9573

This file is one of several utilities developed for the book.

All utilities developed for the book are free software; you can
redistribute and/or modify them under the terms of the GNU
General Public License, Version 2, as published by the Free
Software Foundation.

These utilities are distributed in the hope that they will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the book; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston,
MA  02111-1307  USA

The license is also available on the World Wide Web at
http://www.gnu.org/licenses/gpl.html

*/

//  Common header files
#include "BlasterIncludes.h"

//  Constructor with field number and grammar Element
DataCellX12DT::DataCellX12DT(int iFieldNumberIn,
    const IXMLDOMElementPtr spGrammarIn) :
  DataCellDateYYYYMMDD(iFieldNumberIn, spGrammarIn) { }

//  Destructor
DataCellX12DT::~DataCellX12DT() {};


//  Convert from ISO YYYY-MM-DD to X12 DT data type
int DataCellX12DT::fromXML()
{
  //  Locals
  char cMaxLength[4];
  int iMaxLength, iReturn;

  //  Call base class fromXML method
  iReturn = DataCellDateYYYYMMDD::fromXML();
  if (iReturn != SUCCESS)
  {
    return iReturn;
  }

  //  Maximum Length <- Call Grammar Element's getAttribute for
  //    "MaxLength", oonverting to integer
  memset(cMaxLength,0,sizeof(cMaxLength));
  strcpy(cMaxLength,
      _bstr_t(spGrammar->getAttribute("MaxLength")));
  iMaxLength = atoi(cMaxLength);

  //  IF Maximum Length = "6" for YYMMDD format
  if (iMaxLength == 6)
  {
    //  Shift Cell Buffer two characters to the left
    strcpy(cCellBuffer,&(cCellBuffer[2]));
    cCellBuffer[6] = 0;
    cCellBuffer[7] = 0;
    //  Cell Buffer Length <- 6
    iBufferLength = 6;
  } //  ENDIF
  //  Return success

  return SUCCESS;
}

//  Convert from X12 date data type  to ISO YYYY-MM-DD
int DataCellX12DT::toXML()
{
  //  CellBuffer < trim leading and trailing white space from
  //    CellBuffer
  trim();

  //  IF length of CellBuffer = 6
  if (iBufferLength == 6)
  {
    //  CellBuffer <- "20" + CellBuffer
    //  Cell Buffer Length <- 8
    strcpy(&(cCellBuffer[2]),cCellBuffer);
    strncpy(cCellBuffer,"20",2);
    iBufferLength = 8;
  } //  ENDIF

  //  Return call to base class toXML method
  return DataCellDateYYYYMMDD::toXML();

}
