/* DataCellX12TM.cpp

DataCell derived class for X12 Date data type

From Chapter 9 of
Using XML with Legacy Business Applications
Copyright (c) 2002 Addison-Wesley Professional

Author: Michael C. Rawlins
Rawlins EC Consulting
PMB 29
14 Canyon Creek Village
Richardson, TX 75080-1602
mike@rawlinsecconsulting.com
972-783-9573

This file is one of several utilities developed for the book.

All utilities developed for the book are free software; you can
redistribute and/or modify them under the terms of the GNU
General Public License, Version 2, as published by the Free
Software Foundation.

These utilities are distributed in the hope that they will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the book; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston,
MA  02111-1307  USA

The license is also available on the World Wide Web at
http://www.gnu.org/licenses/gpl.html

*/

//  Common header files
#include "BlasterIncludes.h"

//  Constructor with field number and grammar Element
DataCellX12TM::DataCellX12TM(int iFieldNumberIn,
    const IXMLDOMElementPtr spGrammarIn) :
  DataCell(iFieldNumberIn, spGrammarIn) { }

//  Destructor
DataCellX12TM::~DataCellX12TM() {};


//  Convert from ISO HH:MM:SS.ddddd to X12 TM data type
int DataCellX12TM::fromXML()
{
  //  Locals
  char cTemp[32];
  memset(cTemp,0,sizeof(cTemp));

  //  IF length of CellBuffer < 8
  if (iBufferLength < 8)
  {
    //  Return error
    cerr << BAD_TIME << " " << cCellBuffer << endl;
    return FAILURE;
  } //  ENDIF

  //  Hours <- CellBuffer characters at offsets 0 and 1
  strncpy(cTemp,cCellBuffer,2);
  //  Minutes <- CellBuffer characters at offsets 3 and 4
  strncat(cTemp,&(cCellBuffer[3]),2);
  //  Seconds <- CellBuffer characters at offsets 6 and 7
  strncat(cTemp,&(cCellBuffer[6]),2);
  //  Fractional Seconds <- CellBuffer characters 9 and beyond,
  //  if present
  //  CellBuffer <- Hours + Minutes + Seconds + Fractional
  //    Seconds
  if ( iBufferLength > 8)
  {
    strcat(cTemp,&(cCellBuffer[8]));
  }

  //  Length <- String length of CellBuffer
  //  Return success
  return putField(cTemp,strlen(cTemp));

}


//  Prepare output for legacy format
int DataCellX12TM::prepareOutput()
{
  char cMaxLength[4];
  int iMaxLength;
  memset(cMaxLength,0,sizeof(cMaxLength));

  //  Maximum Length <- Call Grammar Element's getAttribute for
  //    "MaxLength", oonverting to integer
  strcpy(cMaxLength,
      _bstr_t(spGrammar->getAttribute("MaxLength")));
  iMaxLength = atoi(cMaxLength);

  //  IF CellBuffer Length > Maximum length
  if (iBufferLength > iMaxLength)
  {
    //  Trucate Cell Buffer from left to maximum length
    memset(&(cCellBuffer[iMaxLength]),0, (iBufferLength - iMaxLength));
    //  Cell Buffer Length <- Maximum Length
    iBufferLength = iMaxLength;
  }
  //  Return success
  return SUCCESS;
}

//  Convert from X12 TM time data type  to ISO HH:MM:SS.dddd
int DataCellX12TM::toXML()
{
  //  CellBuffer < trim leading and trailing white space from
  //    CellBuffer
  char cTemp[32];
  memset(cTemp,0,sizeof(cTemp));
  trim();

  //  IF length of CellBuffer < 4
  if ( iBufferLength < 4)
  {
    //  Return Error
    cerr << BAD_TIME << " " << cCellBuffer << endl;
    return FAILURE;
  } //  ENDIF

  //  Hours <- CellBuffer characters at offsets 0 and 1
  strncpy(cTemp,cCellBuffer,2);
  strcat(cTemp,":");
  //  Minutes <- CellBuffer characters at offsets 2 and 3
  strncat(cTemp,&(cCellBuffer[2]),2);
  strcat(cTemp,":");
  //  Seconds <- CellBuffer characters at offsets 4 and 5,
  //    if present, otherwise "00"
  if (iBufferLength > 4)
  {
    strncat(cTemp,&(cCellBuffer[4]),2);
  }
  else
  {
    strcat(cTemp,"00");
  }

  //  Fractional Seconds <- CellBuffer characters 6 and beyond
  //  if present
  //  CellBuffer <- Hours + ":" + Minutes + ":" + Seconds + "."
  //    + Fractional Seconds
  if (iBufferLength > 6)
  {
    strcat(cTemp,".");
    strcat(cTemp,&(cCellBuffer[6]));      
  }

  //  Length <- String length of CellBuffer
  //  Return success
  return putField(cTemp,strlen(cTemp));
}
