/* SourceConverter.cpp

Base class for base SourceConverter class

From Chapter 6 of
Using XML with Legacy Business Applications
Copyright (c) 2002 Addison-Wesley Professional

Author: Michael C. Rawlins
Rawlins EC Consulting
PMB 29
14 Canyon Creek Village
Richardson, TX 75080-1602
mike@rawlinsecconsulting.com
972-783-9573

This file is one of several utilities developed for the book.

All utilities developed for the book are free software; you can
redistribute and/or modify them under the terms of the GNU
General Public License, Version 2, as published by the Free
Software Foundation.

These utilities are distributed in the hope that they will be
useful, but WITHOUT ANY WARRANTY; without even the implied
warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with the book; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston,
MA  02111-1307  USA

The license is also available on the World Wide Web at
http://www.gnu.org/licenses/gpl.html

*/

//  Common header files
#include "BlasterIncludes.h"
#include "BlasterPrototypes.h"


//  Special headers for this module only
#include <strstrea.h>


//  Default Constructor
SourceConverter::SourceConverter(bool boValidateIn, 
    const char * const cBaseOutputDirectoryIn) : Converter()
{
#include "BlasterExceptions.h"
  //  Set up implementation dependent environment for loading
  //  and parsing the FileDescription Document

  //  Locals
  //  COM
  int iPathLength;

  //  Validation Option <- Passed validation option
  boValidate = boValidateIn;

  //  Base Output Directory <- From passed base output
  //  directory, Appending directory separator character if
  //  required
  iPathLength = strlen(cBaseOutputDirectoryIn);

  if ( iPathLength > (MAX_PATH_LENGTH - 2) )
  {
    cerr << BUFFER_OVERFLOW_IN << 
      "SourceConverter::SourceConverter" << endl;
    throw cConstructorError;
  }

  memset(cBaseOutputDirectory,0,MAX_PATH_LENGTH);
  strcpy(cBaseOutputDirectory, cBaseOutputDirectoryIn);

  if ( cBaseOutputDirectory[iPathLength - 1] != '\\')
  {
    cBaseOutputDirectory[iPathLength] = '\\';
  }


  //  Perform any implementation specific setup for validating
  //     output documents before writing them
  //  Nothing to do for C++ and MSXML
}


//  Destructor
SourceConverter::~SourceConverter() {};

//  Save the output document
int SourceConverter::saveDocument(
    const IXMLDOMDocument2Ptr spDocOutput,
    const char * const cFileName)
{
  //  Locals

  //  COM
  HRESULT hResult;

  //  XML
  IXMLDOMParseErrorPtr spParseError;

  //  Basic types

  //  Validate if requested
  if (boValidate)
  {
    spParseError = spDocOutput->validate();
    if( spParseError->errorCode != S_OK)
    {
      cerr << VALIDATION_ERROR_BRIEF << cFileName << endl;
      displayParseError(spParseError);
      return FAILURE;
    }
  }

  hResult = spDocOutput->save(cFileName);

  if FAILED(hResult)
  {
    cerr << XML_OUTPUT_SAVE_ERROR << cFileName << endl;
    return FAILURE;
  }

  return SUCCESS;
}

//  Stub functions for derived classes
int SourceConverter::processFile(
    const char * const cInputFileName)
{ return FAILURE; }
